/******************************************************************************
**                            LightSword Alarm Tool                          **
**                            Coded by Mike Farrell                          **
*******************************************************************************
**                      Simple GTK alarm utility........                     **
**                                                                           **
**---------------------------------------------------------------------------**
**                            (C)2003 Mike Farrell                           **
**                     http://ltsword.allegronetwork.com                     **
******************************************************************************/

/*Stuff to do****
* Countdown feature is still pretty inaccurate as it only checks for minutes
*****************/

/*Includes*********************************************************************/
#ifdef WIN32
  #include <windows.h>
  #include <mmsystem.h>
#else
  #include <signal.h>
  #include <sys/types.h>
  #include <sys/stat.h>
  #include <fcntl.h>
#endif
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>
#include <unistd.h>
#include <gtk/gtk.h>
#include "alarm.h"

/*Globals*********************************************************************/
GtkWidget *window, *entry, *entry_hr, *entry_min, *msgbox, *options_dlg,
          *edit, *edit2, *edit3, *browse_button, *check, *check0, *check2,
          *check3, *check4, *check5, *alarm_msg_label, *hour_label, *min_label,
          *combo1;
gchar alarm_msg[2048] = "", important = TRUE, show_popup = TRUE,
      ex_command = FALSE, pm = FALSE, altype = ALTYPE_TOD,
      sound_file[PATH_MAX] = "lsalarm.wav", sound_cmd[256] = "esdplay",
      command_str[256] = "", read_session = FALSE, repeat_sound = FALSE;
gint des_hr = -1, des_min = -1;

/*Functions*******************************************************************/
void check_dirs()
{
  #ifdef __linux
  int fd;
  char buf[512], *homedir = getenv("HOME");

  strcpy(buf, homedir);
  strcat(buf, "/.lsalarm");
  if((fd = open(buf, O_RDWR)) == -1)
  {
    mkdir(buf, 0770);
    strcat(buf, "/sessions");
    mkdir(buf, 0770);
  }
  else
  {
    close(fd);
    strcat(buf, "/sessions");
    if((fd = open(buf, O_RDWR)) == -1)
    {
      mkdir(buf, 0770);
      close(fd);
    }
    else close(fd);
  }
  #endif
}

/*save user preferences*/
void save_settings()
{
  FILE *out;
  char buf[512];

  sprintf(buf, "%s/.lsalarm/config", getenv("HOME"));
  out = fopen(buf, "w");

  if(out)
  {
    fprintf(out, "## Configuration settings for LightSword Alarm Clock\n");
    fprintf(out, "## Lsalarm version %s\n\n", VERSION_STR);
    fprintf(out, "sound_file=%s\n", sound_file);
    fprintf(out, "play_sound=%d\n", (int)important);
    fprintf(out, "sound_command=%s\n", sound_cmd);
    fclose(out);
  }
}

/*load user preferences*/
void load_settings()
{
  FILE *out;
  char buf[512];

  sprintf(buf, "%s/.lsalarm/config", getenv("HOME"));
  out = fopen(buf, "r");

  if(out)
  {
    while(fgets(buf, 512, out))
    {
      if(strstr(buf, "sound_file"))
        sscanf(buf, "sound_file=%s", sound_file);
      else if(strstr(buf, "sound_command"))
        sscanf(buf, "sound_command=%s", sound_cmd);
      else if(strstr(buf, "play_sound"))
      {
        int tmp;

        sscanf(buf, "play_sound=%d", &tmp);
        important = tmp;
      }
    }
    fclose(out);
  }
}


/*Generate a temporary file to show that process is still alive*/
void response_request()
{
  #ifdef __linux
  char buf[PATH_MAX];
  FILE *file = NULL;

  sprintf(buf, "%s/.lsalarm/%d", getenv("HOME"), getpid());
  file = fopen(buf, "w");
  fputc('1', file);
  fclose(file);
  #endif
}

/*File generation routine to store info for every lsalarm session*/
void status_request()
{
  #ifdef linux
  char buf[PATH_MAX];
  FILE *file = NULL;

  sprintf(buf, "%s/.lsalarm/sessions/lsalarm-info.%d.txt", getenv("HOME"), getpid());

  file = fopen(buf, "w");
  if(file)
  {
    fprintf(file, "--LsAlarm session info (%d)....--\n", getpid());
    fprintf(file, "Waiting for alarm at:  %.2d:%.2d\n", des_hr, des_min);
    fprintf(file, "Alarm str is:  %s\n", alarm_msg);
    fprintf(file, "Sound file is:  %s\n", sound_file);
    fprintf(file, "Command str is:  %s\n", command_str);
    fprintf(file, "Repeating: %d\n", (int)repeat_sound);

    fclose(file);
  }
  #endif
}

/*Restore an alarm from an existing session file
  making sure it's dead, first*/
void run_session_file(char *path)
{
  #ifdef linux
  FILE *file = fopen(path, "r");
  char line[512];

  printf("Trying to read session file...\n");
  if(file)
  {
    int ps_id, sound_tmp, i;

    while(fgets(line, 512, file))
    {
      if(sscanf(line, "--LsAlarm session info (%d)....--", &ps_id) == 1)
      {
        int ret = process_exists(ps_id);

        if(ret == 1)
          printf("Warning:  Pid %d exists!!!  Might be a duplicate\n", ps_id);
        else if(ret == 0 && strstr(path, ".lsalarm/sessions/"))
          remove(path);
      }
      else if(sscanf(line, "Waiting for alarm at:  %d:%d", &des_hr, &des_min) == 2)
        printf("Successfully read destination time %d:%d\n", des_hr, des_min);
      else if(strstr(line, "Alarm str is"))
      {
        i = 0;
        while(line[++i] != ':') ;

        strcpy(alarm_msg, line+i+3);
        for(i = 0; alarm_msg[i]; i++)
        {
          if(alarm_msg[i] == '\n')
            alarm_msg[i] = 0;
        }
        printf("Successfully read Alarm str (%s)\n", alarm_msg);
      }
      else if(strstr(line, "Sound file is"))
      {
        i = 0;
        while(line[++i] != ':') ;

        strcpy(sound_file, line+i+3);
        for(i = 0; sound_file[i]; i++)
        {
          if(sound_file[i] == '\n')
            sound_file[i] = 0;
        }
        printf("Successfully read Sound file (%s)\n", sound_file);
      }
      else if(strstr(line, "Command str is"))
      {
        i = 0;
        while(line[++i] != ':') ;

        strcpy(command_str, line+i+3);
        for(i = 0; command_str[i]; i++)
        {
          if(command_str[i] == '\n')
            command_str[i] = 0;
        }
        if(command_str[0] != 0)
          ex_command = TRUE;
        printf("Successfully read Command str (%s)\n", command_str);
      }
      else if(sscanf(line, "Repeating:  %d\n", &sound_tmp))
      {
        repeat_sound = sound_tmp;
        printf("Successfully read repeating %d\n", (int)repeat_sound);
      }
    }

    if(des_hr == 0)
      des_hr = 12;
    else if(des_hr >= 12)
    {
      if(des_hr >= 13)
        des_hr -= 12;

      pm = TRUE;
    }

    read_session = TRUE;
  }
  else
    read_session = FALSE;
  #endif
}

void cleanup()
{
  #ifdef linux
  char buf[PATH_MAX];

  sprintf(buf, "%s/.lsalarm/sessions/lsalarm-info.%d.txt", getenv("HOME"), getpid());
  remove(buf);
  #endif

  save_settings();
}

void terminate()
{
  cleanup();
  exit(0);
}

/*Do the sound event*/
void play_sound()
{
  char buf[PATH_MAX], *sound_dir = "../share/lsalarm/sounds/";
  int i = 0;

  /*look for an absolute path to sound file*/
  while(sound_file[i] && i < PATH_MAX-1)
  {
    if(sound_file[i] == '/' || sound_file[i] == '\\')
    {
      sound_dir = "";
      break;
    }
    i++;
  }

  /*play the sound*/
  sprintf(buf, "%s %s%s > /dev/null", sound_cmd, sound_dir, sound_file);
  printf("Sound: %s\n", buf);
  #ifndef WIN32
    system(buf);
  #else
    PlaySound(sound_file, NULL, SND_NOWAIT | SND_ASYNC);
  #endif
}

/*Test the sound from the options dlg*/
void test_sound()
{
  strcpy(sound_file, gtk_entry_get_text(GTK_ENTRY(edit)));
  strcpy(sound_cmd, gtk_entry_get_text(GTK_ENTRY(edit2)));
  play_sound();
}

/*Callback for repeating sounds*/
gint sound_repeat_callback(gpointer data)
{
  play_sound();

  return TRUE;
}


/*Callback to close the msgbox window*/
void close_msgbox()
{
  //gtk_timeout_remove(sound_repeat_callback);
  gtk_widget_hide(msgbox);
}

/*Put up an alarm message or whatever*/
void do_alarm(gchar *override_msg)
{
  GtkWidget *button;
  GtkWidget *table;
  GtkWidget *label;
  GtkWidget *text;
  gint w = 300, h = 100;

  printf("Status: Alarm point reached\n");
  if(!override_msg)
  {
    if(important)
    {
      play_sound();
      if(repeat_sound)
        gtk_timeout_add(4000, sound_repeat_callback, NULL);
    }

    if(ex_command)
    {
      char buf[512];
      FILE *file;

      /*I've found no better way to do this..*/
      #ifdef __linux
        sprintf(buf, "%s > ~/.lsalarm/tmp.lsalarm", command_str);
      #else
        sprintf(buf, "%s > tmp.lsalarm", command_str);
      #endif
      printf("Command: %s\n", command_str);
      system(buf);
      #ifdef __linux
        sprintf(buf, "%s/.lsalarm/tmp.lsalarm", getenv("HOME"));
      #else
        strcpy(buf, "tmp.lsalarm");
      #endif
      file = fopen(buf, "r");
      if(!file)
      {
        fprintf(stderr, "Can't open ~/.lsalarm/tmp.lsalarm\n");
        return;
      }
      alarm_msg[0] = 0;
      while(fgets(buf, 512, file))
      {
        if(strlen(alarm_msg)+strlen(buf) < 2048)
          strcat(alarm_msg, buf);
        else
        {
          int i, pos = 0;

          for(i = strlen(alarm_msg)-1; (i < 2047 && buf[pos]); i++)
          {
            alarm_msg[i] = buf[pos];
            pos++;
          }
          alarm_msg[i] = 0;
        }
      }
      h = 200;
    }
  }

  /*put up a msg box here*/
  msgbox = gtk_window_new(GTK_WINDOW_TOPLEVEL);

  //gtk_widget_set_usize(GTK_WIDGET (msgbox), w, h);
  gtk_window_set_position(GTK_WINDOW (msgbox), GTK_WIN_POS_CENTER);
  gtk_window_set_title(GTK_WINDOW (msgbox), "ALAAAARM!");
  gtk_container_set_border_width(GTK_CONTAINER (msgbox), 20);

  table = gtk_table_new(2, 2, TRUE);
  gtk_container_add(GTK_CONTAINER(msgbox), table);

  if(strcmp(alarm_msg, "") == 0) strcpy(alarm_msg, "(none)");
  if(override_msg) strcpy(alarm_msg, override_msg);
  if(!override_msg && ex_command)
  {
    text = gtk_text_new (NULL, NULL);
    gtk_table_attach(GTK_TABLE (table), text, 0, 2, 0, 1, (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL), 0, 0);
    gtk_text_insert (text, NULL, &text->style->black, NULL, alarm_msg, -1);
    gtk_widget_show(text);
    w += 200;
    h += 100;
    gtk_widget_set_usize(GTK_WIDGET (msgbox), w, h);
    gtk_window_set_position(GTK_WINDOW (msgbox), GTK_WIN_POS_CENTER);
  }
  else
  {
    label = gtk_label_new(alarm_msg);
    gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
    gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 2, 0, 1);
    gtk_widget_show(label);
  }
  gtk_window_set_policy (GTK_WINDOW(msgbox), FALSE, FALSE, FALSE);

  button = gtk_button_new_with_label("Okay");
  gtk_widget_set_usize(button, -1, 35);
  if(!override_msg) gtk_signal_connect(GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC(gtk_exit), (gpointer) "Okay");
  else gtk_signal_connect(GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC(close_msgbox), (gpointer) "Okay");
  gtk_table_attach(GTK_TABLE (table), button, 0, 1, 1, 2, (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL), 0, 0);
  gtk_widget_show(button);

  button = gtk_button_new_with_label("Cancel");
  gtk_widget_set_usize(button, -1, 35);
  if(!override_msg) gtk_signal_connect(GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC (gtk_exit), (gpointer) "Cancel");
  else gtk_signal_connect(GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC(close_msgbox), (gpointer) "Cancel");
  gtk_table_attach(GTK_TABLE (table), button, 1, 2, 1, 2, (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL), 0, 0);

  if(!override_msg) gtk_signal_connect(GTK_OBJECT (msgbox), "delete_event", GTK_SIGNAL_FUNC (gtk_exit), NULL);
  else gtk_signal_connect(GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC(close_msgbox), (gpointer) "Okay");

  gtk_widget_show(button);
  gtk_widget_show(table);

  if(override_msg || show_popup)
    gtk_widget_show(msgbox);
  else gtk_main_quit();
}

/*Timer Callback function called once every minute*/
gint timer_callback(gpointer data)
{
  gint ret = TRUE;
  time_t d;
  struct tm *t;

  time(&d);
  t = localtime(&d);

  if(t->tm_hour == des_hr && t->tm_min == des_min)
  {
    do_alarm(NULL);
    ret = FALSE;
  }
  return ret;
}

/*Toggle whether to play a sound or not*/
void entry_toggle_important(GtkWidget *checkbutton, GtkWidget *entry)
{
  if(GTK_WIDGET_IS_SENSITIVE(edit))
    gtk_widget_set_sensitive(edit, FALSE);
  else gtk_widget_set_sensitive(edit, TRUE);
  if(GTK_WIDGET_IS_SENSITIVE(edit2))
    gtk_widget_set_sensitive(edit2, FALSE);
  else gtk_widget_set_sensitive(edit2, TRUE);
  if(GTK_WIDGET_IS_SENSITIVE(browse_button))
    gtk_widget_set_sensitive(browse_button, FALSE);
  else gtk_widget_set_sensitive(browse_button, TRUE);
}

void toggle_repeat(GtkWidget *checkbutton, GtkWidget *entry)
{
  repeat_sound = !repeat_sound;
}

/*Toggle whether to execute a command or not*/
void toggle_command(GtkWidget *checkbutton)
{
  if(GTK_WIDGET_IS_SENSITIVE(edit3))
    gtk_widget_set_sensitive(edit3, FALSE);
  else gtk_widget_set_sensitive(edit3, TRUE);
}

/*This function sets the alarm*/
void button_clicked()
{
  char min_out = 0;
  const char *combo_text = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(combo1)->entry));


  if(!read_session)
  {
    strcpy(alarm_msg, gtk_entry_get_text(GTK_ENTRY(entry)));
    des_hr = atoi(gtk_entry_get_text(GTK_ENTRY(entry_hr)));
    des_min = atoi(gtk_entry_get_text(GTK_ENTRY(entry_min)));
    pm = (strcmp(combo_text, "P.M.") == 0);
  }

  if(altype == ALTYPE_TOD)
    min_out = (des_min < 0 || des_min > 59);
  else
    min_out = (des_min <= 0 || des_min > 1440);

  if((des_hr != -1 && (des_hr < 1 || des_hr > 12)) || min_out)
    do_alarm("That isn't very smart :-) (data out of range)");
  else
  {
    /*Calculate a destination for countdown*/
    if(altype == ALTYPE_COUNTDOWN)
    {
      struct tm *t;
      time_t d;
      int hour_offset, min_offset;

      time(&d);
      t = localtime(&d);

      hour_offset = des_min/60;
      min_offset = des_min%60;

      des_hr = t->tm_hour+hour_offset;
      des_min = t->tm_min+min_offset;

      if(des_min >= 60)
      {
        des_min %= 60;
        des_hr++;
      }
      des_hr %= 24;

      printf("Countdown conversion in effect...\n");
    }
    /*-1 is for this current hour*/
    else if(des_hr == -1)
    {
      struct tm *t;
      time_t d;

      time(&d);
      t = localtime(&d);
      des_hr = t->tm_hour;

      printf("Using current hour: %d\n", des_hr);
    }
    else if(!read_session)
    {
      if(pm)
      {
        /*calculate military time from standard*/
        des_hr += 12;

        if(des_hr == 24)
          des_hr = 12;


        printf("PM conversion in effect...\n");
      }
      /*am*/
      else
      {
        if(des_hr == 12)
          des_hr = 0;
      }
    }

    printf("Status:  Alarm Entry contents = %s\n", alarm_msg);
    printf("Got final destination time as: %d:%.2d\n", des_hr, des_min);
    printf("Waiting for alarm time......\n");
    gtk_timeout_add(TIMER_INTERVAL, timer_callback, NULL);
    gtk_widget_hide(window);
  }

  //generate session info file
  status_request();
}

/*Cancel all option changes*/
void opt_cancel()
{
  gtk_widget_hide(options_dlg);
  gtk_widget_show(window);
}

/*Apply all option changes*/
void opt_ok()
{
  gchar oldtype = altype;

  /*Update settings*/
  show_popup = (check2->state == GTK_STATE_ACTIVE);
  important = (check3->state == GTK_STATE_ACTIVE);
  ex_command = (check4->state == GTK_STATE_ACTIVE);
  altype = (check->state == GTK_STATE_ACTIVE) ? ALTYPE_TOD : ALTYPE_COUNTDOWN;

  /*Reset the main window layout if necessary*/
  if(oldtype != altype)
  {
    if(altype == ALTYPE_COUNTDOWN)
    {
      gtk_widget_hide(GTK_WIDGET(hour_label));
      gtk_widget_hide(GTK_WIDGET(entry_hr));
      gtk_widget_hide(GTK_WIDGET(combo1));
      gtk_label_set_text(GTK_LABEL(min_label), "Minutes until: ");
    }
    else
    {
      gtk_widget_show(GTK_WIDGET(hour_label));
      gtk_widget_show(GTK_WIDGET(entry_hr));
      gtk_widget_show(GTK_WIDGET(combo1));
      gtk_label_set_text(GTK_LABEL(min_label), "Minute (60): ");
    }
  }

  strcpy(sound_file, gtk_entry_get_text(GTK_ENTRY(edit)));
  strcpy(sound_cmd, gtk_entry_get_text(GTK_ENTRY(edit2)));
  strcpy(command_str, gtk_entry_get_text(GTK_ENTRY(edit3)));

  /*call this to close the window and save code*/
  opt_cancel();
}

/*Callback for file selection OK button*/
void sound_selected(GtkWidget *widget, GtkWidget *win)
{
  gtk_entry_set_text(GTK_ENTRY(edit), gtk_file_selection_get_filename(GTK_FILE_SELECTION(win)));
  gtk_widget_destroy(win);
}

/*Look for a sound to play*/
void browse_files()
{
  GtkWidget *filew;

  filew = gtk_file_selection_new ("Pick a WAV file");
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION(filew)->ok_button), "clicked", (GtkSignalFunc) sound_selected, filew);
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(filew)->cancel_button), "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (filew));

  gtk_file_selection_complete(GTK_FILE_SELECTION(filew), "../share/lsalarm/sounds/");

  gtk_widget_show(filew);
}

/*Show the option dialog*/
void show_options()
{
  GtkWidget *button;
  GtkWidget *table;
  GtkWidget *label, *vbox, *vbox2, *vbox3, *hbox;
  GtkWidget *mainframe, *altype_frame, *snd_frame;
  GSList *group;

  gtk_widget_hide(window);

  /*Put up a dialog box with options*/
  options_dlg = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  //gtk_widget_set_usize(GTK_WIDGET (options_dlg), 320, 290);
  gtk_window_set_position(GTK_WINDOW (options_dlg), GTK_WIN_POS_CENTER);
  gtk_window_set_policy (GTK_WINDOW(options_dlg), FALSE, FALSE, FALSE);
  gtk_window_set_title(GTK_WINDOW (options_dlg), "Configure LightSword Alarm");
  gtk_signal_connect(GTK_OBJECT (options_dlg), "delete_event", (GtkSignalFunc) opt_cancel, NULL);
  gtk_container_set_border_width(GTK_CONTAINER (options_dlg), 5);

  vbox = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER (options_dlg), vbox);
  gtk_widget_show(vbox);

  /*Main frame*/
  mainframe = gtk_frame_new("Options");
  gtk_box_pack_start(GTK_BOX(vbox), mainframe, TRUE, TRUE, 0);
  gtk_widget_show(mainframe);

  vbox2 = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER (mainframe), vbox2);
  gtk_widget_show(vbox2);
  gtk_box_set_homogeneous(GTK_BOX(vbox2), FALSE);

  /*Alarm type options frame*/
  altype_frame = gtk_frame_new("Alarm type");
  gtk_box_pack_start(GTK_BOX(vbox2), altype_frame, TRUE, TRUE, 0);
  gtk_container_set_border_width(GTK_CONTAINER (altype_frame), 5);
  gtk_widget_show(altype_frame);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(altype_frame), hbox);
  gtk_widget_show(hbox);

  check = gtk_radio_button_new_with_label (NULL, "Time of day");
  gtk_box_pack_start(GTK_BOX(hbox), check, TRUE, TRUE, 0);
  if(altype == ALTYPE_TOD) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), TRUE);
  gtk_widget_show(check);

  /*Using a check0, because I stupidly forgot to add this one*/
  group = gtk_radio_button_group (GTK_RADIO_BUTTON (check));
  check0 = gtk_radio_button_new_with_label (group, "Countdown");
  gtk_box_pack_start(GTK_BOX(hbox), check0, TRUE, TRUE, 0);
  if(altype == ALTYPE_COUNTDOWN) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check0), TRUE);
  gtk_widget_show(check0);

  /*Sound options frame*/
  snd_frame = gtk_frame_new("Alarm Options");
  gtk_box_pack_start(GTK_BOX(vbox2), snd_frame, TRUE, TRUE, 0);
  gtk_container_set_border_width(GTK_CONTAINER (snd_frame), 5);
  gtk_widget_show(snd_frame);

  vbox3 = gtk_vbox_new(FALSE, 1);
  gtk_container_add(GTK_CONTAINER(snd_frame), vbox3);
  gtk_widget_show(vbox3);

  hbox = gtk_hbox_new(FALSE, 2);
  gtk_box_pack_start(GTK_BOX(vbox3), hbox, TRUE, TRUE, 0);
  gtk_widget_show(hbox);

  check2 = gtk_check_button_new_with_label("Popup notification");
  gtk_box_pack_start(GTK_BOX (hbox), check2, TRUE, TRUE, 0);
  if(show_popup) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check2), TRUE);
  gtk_widget_show(check2);

  check3 = gtk_check_button_new_with_label("Play sound");
  gtk_box_pack_start(GTK_BOX (hbox), check3, TRUE, TRUE, 0);
  if(important) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check3), TRUE);
  gtk_signal_connect(GTK_OBJECT(check3), "toggled", GTK_SIGNAL_FUNC(entry_toggle_important), entry);
  gtk_widget_show(check3);

  hbox = gtk_hbox_new(FALSE, 2);
  gtk_box_pack_start(GTK_BOX(vbox3), hbox, TRUE, TRUE, 0);
  gtk_widget_show(hbox);

  check4 = gtk_check_button_new_with_label("Execute command");
  gtk_box_pack_start(GTK_BOX (hbox), check4, TRUE, TRUE, 0);
  if(ex_command) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check4), TRUE);
  gtk_signal_connect(GTK_OBJECT(check4), "toggled", GTK_SIGNAL_FUNC(toggle_command), NULL);
  gtk_widget_show(check4);

  check5 = gtk_check_button_new_with_label("Repeat sound");
  gtk_box_pack_start(GTK_BOX (hbox), check5, TRUE, TRUE, 0);
  if(repeat_sound) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check5), TRUE);
  gtk_signal_connect(GTK_OBJECT(check5), "toggled", GTK_SIGNAL_FUNC(toggle_repeat), entry);
  gtk_widget_show(check5);

  hbox = gtk_hbox_new(FALSE, 2);
  gtk_box_pack_start(GTK_BOX(vbox3), hbox, TRUE, TRUE, 0);
  gtk_widget_show(hbox);

  label = gtk_label_new("Alert sound");
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
  gtk_widget_show(label);

  edit = gtk_entry_new_with_max_length(256);
  gtk_widget_set_usize(GTK_WIDGET (edit), 140, 20);
  if(STR_EMPTY(sound_file))
    gtk_entry_set_text(GTK_ENTRY(edit), "lsalarm.wav");
  else gtk_entry_set_text(GTK_ENTRY(edit), sound_file);
  gtk_box_pack_start(GTK_BOX (hbox), edit, FALSE, FALSE, 2);
  gtk_widget_show(edit);

  browse_button = gtk_button_new_with_label("Browse");
  gtk_box_pack_start(GTK_BOX (hbox), browse_button, FALSE, FALSE, 2);
  gtk_signal_connect(GTK_OBJECT(browse_button), "clicked", GTK_SIGNAL_FUNC(browse_files), NULL);
  gtk_widget_show(browse_button);

  hbox = gtk_hbox_new(FALSE, 2);
  gtk_box_pack_start(GTK_BOX(vbox3), hbox, TRUE, TRUE, 2);
  gtk_widget_show(hbox);

  label = gtk_label_new("Sound command");
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
  gtk_widget_show(label);

  edit2 = gtk_entry_new_with_max_length(256);
  if(STR_EMPTY(sound_cmd))
    gtk_entry_set_text(GTK_ENTRY(edit2), "esdplay");
  else gtk_entry_set_text(GTK_ENTRY(edit2), sound_cmd);
  gtk_box_pack_start(GTK_BOX (hbox), edit2, FALSE, FALSE, 2);
  gtk_widget_set_usize(edit2, 95, -1);
  gtk_widget_show(edit2);

  button = gtk_button_new_with_label("Test Sound");
  gtk_signal_connect(GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC(test_sound), (gpointer) "Test Sound");
  gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 2);
  gtk_widget_show(button);

  hbox = gtk_hbox_new(FALSE, 2);
  gtk_box_pack_start(GTK_BOX(vbox3), hbox, TRUE, TRUE, 2);
  gtk_widget_show(hbox);

  label = gtk_label_new("Command to execute");
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
  gtk_widget_show(label);

  edit3 = gtk_entry_new_with_max_length(256);
  gtk_widget_set_usize(GTK_WIDGET (edit3), 140, 20);
  gtk_entry_set_text(GTK_ENTRY (edit3), command_str);
  gtk_box_pack_start(GTK_BOX (hbox), edit3, FALSE, FALSE, 2);
  gtk_widget_show(edit3);

  /*Ok and cancel buttons*/
  table = gtk_table_new(2, 2, TRUE);
  gtk_table_set_col_spacings(GTK_TABLE(table), 5);
  gtk_box_pack_start(GTK_BOX (vbox), table, FALSE, TRUE, 0);
  gtk_widget_set_usize(GTK_WIDGET (table), 300, 50);

  button = gtk_button_new_with_label("Okay");
  gtk_signal_connect(GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC(opt_ok), (gpointer) "Okay");
  gtk_table_attach_defaults (GTK_TABLE(table), button, 0, 1, 1, 2);
  gtk_widget_show(button);

  button = gtk_button_new_with_label("Cancel");
  gtk_signal_connect(GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC (opt_cancel), (gpointer) "Cancel");
  gtk_table_attach_defaults(GTK_TABLE(table), button, 1, 2, 1, 2);
  gtk_widget_show(button);
  gtk_widget_show(table);

  /*Disable widgets based off of variables*/
  if(!important)
  {
    gtk_widget_set_sensitive(edit, FALSE);
    gtk_widget_set_sensitive(edit2, FALSE);
    gtk_widget_set_sensitive(browse_button, FALSE);
  }
  if(!ex_command)
    gtk_widget_set_sensitive(edit3, FALSE);

  gtk_widget_show(options_dlg);
}

/*Return 1 if process exists, 0 if not, -1 if unknown (win32)
  I have no idea if this will work on non-Linux unix platforms*/
int process_exists(int pid)
{
  #ifdef WIN32
    return -1;
  #else

  int ret = 0;
  char path[PATH_MAX];
  struct stat buf;

  sprintf(path, "/proc/%d", pid);
  if(stat(path, &buf) != -1 && S_ISDIR(buf.st_mode))
    ret = 1;
    
  return ret;
  #endif
}

/*Main Entrypoint*/
int main(int argc, char *argv[])
{
  GtkWidget *vbox, *hbox;
  GtkWidget *button, *opt_button;
  GtkWidget *label;
  int i;
  char own_style = FALSE, buf[64];
  GList *combo1_strings = NULL;
  time_t d;
  struct tm *t;

  /*Siguser handler*/
  #ifndef WIN32
    signal(SIGUSR1, status_request);
    signal(SIGUSR2, response_request);
    signal(SIGTERM, terminate);
    signal(SIGINT, terminate);
  #else
    /*win32 uses ownstyle by default*/
    own_style = TRUE;
  #endif

  /*Make sure all directories needed exist and load settings*/
  check_dirs();
  load_settings();

  /**set current hour to now**/
  time(&d);
  t = localtime(&d);
  des_hr = t->tm_hour;

  if(des_hr >= 12)
  {
    if(des_hr > 12)
      des_hr %= 12;
    pm = TRUE;
  }

  /*special case for 12-am*/
  if(des_hr == 0)
  {
    des_hr = 12;
    pm = FALSE;
  }


  combo1_strings = g_list_append(combo1_strings, (void *)"A.M.");
  combo1_strings = g_list_append(combo1_strings, (void *)"P.M.");

  /*Handle command line args*/
  for(i = 1; i < argc; i++)
  {
    if(strcmp(argv[i], "-ownstyle") == 0)
      own_style = TRUE;
    else if(strstr(argv[i], ".txt") && !read_session)
      run_session_file(argv[i]);
  }

  /*Make sure program is run from it's own directory*/
  #ifndef WIN32
    chdir(PREFIX);
    chdir("bin");
  #endif


  printf("-LightSword Alarm Clock Version %s-\n\n", VERSION_STR);
  gtk_init(&argc, &argv);
  if(own_style)
  {
    printf("Forcing default gtk style..\n");
    gtk_rc_parse("gtkrc");
  }

  /*Initialize and show main window*/
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_usize(GTK_WIDGET (window), 200, -1);
  gtk_window_set_position(GTK_WINDOW (window), GTK_WIN_POS_CENTER);
  gtk_window_set_policy (GTK_WINDOW(window), FALSE, FALSE, TRUE);
  gtk_window_set_title(GTK_WINDOW (window), "LightSword Alarm Clock");
  gtk_signal_connect(GTK_OBJECT (window), "delete_event", (GtkSignalFunc) gtk_exit, NULL);

  vbox = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER (window), vbox);
  gtk_widget_show(vbox);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER (vbox), hbox);
  gtk_widget_show(hbox);

  label = gtk_label_new("Alarm msg: ");
  gtk_box_pack_start(GTK_BOX (hbox), label, FALSE, FALSE, 5);
  gtk_widget_show(label);

  entry = gtk_entry_new_with_max_length(256);
  gtk_signal_connect(GTK_OBJECT(entry), "activate", GTK_SIGNAL_FUNC(button_clicked), entry);
  gtk_entry_set_text(GTK_ENTRY (entry), "...");
  gtk_entry_select_region(GTK_ENTRY (entry), 0, GTK_ENTRY(entry)->text_length);
  gtk_box_pack_start(GTK_BOX (hbox), entry, TRUE, TRUE, 0);
  gtk_widget_show(entry);

  hbox = gtk_hbox_new(FALSE, 5);
  gtk_container_add(GTK_CONTAINER (vbox), hbox);
  gtk_widget_show(hbox);

  hour_label = gtk_label_new("Hour (12):  ");
  gtk_box_pack_start(GTK_BOX (hbox), hour_label, FALSE, TRUE, 5);
  gtk_widget_show(hour_label);

  sprintf(buf, "%d", des_hr);
  entry_hr = gtk_entry_new_with_max_length(2);
  gtk_entry_set_text(GTK_ENTRY (entry_hr), buf);
  gtk_box_pack_start(GTK_BOX (hbox), entry_hr, FALSE, FALSE, 0);
  gtk_widget_set_usize(entry_hr, 30, -1);
  gtk_widget_show(entry_hr);

  combo1 = gtk_combo_new();
  gtk_combo_set_popdown_strings(GTK_COMBO(combo1), combo1_strings);
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(combo1)->entry), FALSE);
  if(pm)
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(combo1)->entry), "P.M.");
  gtk_widget_set_usize(combo1, 60, -1);
  gtk_box_pack_start(GTK_BOX(hbox), combo1, FALSE, FALSE, 0);
  gtk_widget_show(combo1);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER (vbox), hbox);
  gtk_widget_show(hbox);

  min_label = gtk_label_new("Minute (60): ");
  gtk_box_pack_start(GTK_BOX (hbox), min_label, FALSE, TRUE, 5);
  gtk_widget_show(min_label);

  entry_min = gtk_entry_new_with_max_length(4);
  gtk_entry_set_text(GTK_ENTRY (entry_min), "0");
  gtk_box_pack_start(GTK_BOX (hbox), entry_min, FALSE, FALSE, 0);
  gtk_widget_set_usize(entry_min, 40, -1);
  gtk_widget_show(entry_min);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER (vbox), hbox);
  gtk_widget_show(hbox);

  opt_button = gtk_button_new_with_label("Options");
  gtk_widget_set_usize(GTK_WIDGET(opt_button), -1, 30);
/*  #ifndef WIN32
    gtk_widget_set_uposition(GTK_WIDGET(opt_button), 135, 66);
  #else
    gtk_widget_set_uposition(GTK_WIDGET(opt_button), 135, 72);
  #endif*/
  gtk_widget_set_usize(GTK_WIDGET(opt_button), 65, 22);
  gtk_box_pack_end(GTK_BOX(hbox), opt_button, FALSE, FALSE, 0);
  gtk_signal_connect(GTK_OBJECT(opt_button), "clicked", GTK_SIGNAL_FUNC(show_options), GTK_OBJECT(window));
  gtk_widget_show(opt_button);

  button = gtk_button_new_with_label("Commence");
  gtk_signal_connect_object(GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC(button_clicked), GTK_OBJECT (window));
  gtk_box_pack_start(GTK_BOX (vbox), button, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
  gtk_widget_grab_default(button);
  gtk_widget_show (button);

  if(!read_session)
    gtk_widget_show(window);
  else
    button_clicked();

  atexit(cleanup);
  gtk_main();

  return 0;
}


